/********************************************************************************
 * Copyright (c) 2018 AMFD GmbH
 *               2018-2019 in-tech GmbH
 *               2023 Bayerische Motoren Werke Aktiengesellschaft (BMW AG)
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 ********************************************************************************/

/**
 * \brief Provides all vehicle parameters for the driver
 *
 */

#pragma once

#include <sstream>
#include <string>
#include <vector>

#include "common/globalDefinitions.h"
#include "include/signalInterface.h"

/*!
 * \brief Provides all vehicle parameters for the driver
 */

class ParametersVehicleSignal : public SignalInterface
{
public:
  /// component name as ParametersAgentMachineSignal
  const std::string COMPONENTNAME = "ParametersAgentMachineSignal";

  /**
   * @brief Construct a new Parameters Vehicle Signal object
   *
   * @param vehicleParameters Vehicle parameters
   */
  ParametersVehicleSignal(mantle_api::VehicleProperties vehicleParameters) : vehicleParameters(vehicleParameters) {}

  virtual ~ParametersVehicleSignal() = default;

  //-----------------------------------------------------------------------------
  //! Returns the content/payload of the signal as an std::string
  //!
  //! @return                       Content/payload of the signal as an std::string
  //-----------------------------------------------------------------------------
  virtual operator std::string() const
  {
    std::ostringstream stream;
    stream << COMPONENTNAME;
    return stream.str();
  }

  /// vehicle model parameters
  mantle_api::VehicleProperties vehicleParameters;
};
