/********************************************************************************
 * Copyright (c) 2020 HLRS, University of Stuttgart
 *               2019-2021 in-tech GmbH
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 ********************************************************************************/

//-----------------------------------------------------------------------------
//! @brief This file contains all functions for class SensorDataSignal
//!
//! This class contains all functionality of the module.
//-----------------------------------------------------------------------------

#pragma once

#include <osi3/osi_sensordata.pb.h>
#include <sstream>
#include <string>

#include "include/signalInterface.h"

class SensorDataSignal : public SignalInterface
{
public:
  /// Constructor
  const std::string COMPONENTNAME = "SensorDataSignal";

  /**
   * @brief Construct a new Sensor Data Signal object
   *
   * @param sensorData OSI sensor data
   */
  SensorDataSignal(osi3::SensorData sensorData) : sensorData(sensorData) {}

  SensorDataSignal(const SensorDataSignal&) = delete;
  SensorDataSignal(SensorDataSignal&&) = delete;
  SensorDataSignal& operator=(const SensorDataSignal&) = delete;
  SensorDataSignal& operator=(SensorDataSignal&&) = delete;

  virtual ~SensorDataSignal() {}

  //-----------------------------------------------------------------------------
  //! Returns the content/payload of the signal as an std::string
  //!
  //! @return     Content/payload of the signal as an std::string
  //-----------------------------------------------------------------------------
  virtual operator std::string() const
  {
    std::ostringstream stream;
    stream << COMPONENTNAME;
    return stream.str();
  }
  /// OSI SensorData payload of this signal
  osi3::SensorData sensorData;
};
