/********************************************************************************
 * Copyright (c) 2016-2018 ITK Engineering GmbH
 *               2017-2019 in-tech GmbH
 *               2023 Bayerische Motoren Werke Aktiengesellschaft (BMW AG)
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 ********************************************************************************/

#include "stochasticsBinding.h"

#include <new>

#include "bindings/stochasticsLibrary.h"

class CallbackInterface;
class StochasticsInterface;

namespace core
{

StochasticsBinding::StochasticsBinding(CallbackInterface* callbacks) : callbacks(callbacks) {}

StochasticsBinding::~StochasticsBinding()
{
  Unload();
}

StochasticsInterface* StochasticsBinding::Instantiate(const std::string& libraryPath)
{
  if (library == nullptr)
  {
    try
    {
      library = std::make_shared<StochasticsLibrary>(libraryPath, callbacks);
    }
    catch (const std::bad_alloc&)
    {
      library = nullptr;
      return nullptr;
    }

    if (!library->Init())
    {
      return nullptr;
    }
  }

  return library->CreateStochastics();
}

void StochasticsBinding::Unload()
{
  if (library != nullptr)
  {
    library->ReleaseStochastics();
    library = nullptr;
  }
}

}  // namespace core
