/********************************************************************************
 * Copyright (c) 2017-2018 ITK Engineering GmbH
 *               2017-2021 in-tech GmbH
 *               2022 Bayerische Motoren Werke Aktiengesellschaft (BMW AG)
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 ********************************************************************************/

#pragma once

#include <stdint.h>
#include <string>
#include <units.h>

#include "include/roadInterface/roadElementTypes.h"
#include "include/roadInterface/roadObjectInterface.h"

class RoadInterface;

/// Class representing a road object
class RoadObject : public RoadObjectInterface
{
public:
  //! RoadObject constructor
  //!
  //! @param[in] road     The RoadInterface input data
  //! @param[in] object   OpenDRIVE road objects element
  RoadObject(RoadInterface* road, const RoadObjectSpecification object) : road{road}, object{object} {}

  std::string GetId() const override;
  units::length::meter_t GetS() const override;
  units::length::meter_t GetT() const override;
  units::length::meter_t GetZOffset() const override;
  units::length::meter_t GetHeight() const override;
  units::angle::radian_t GetHdg() const override;
  units::angle::radian_t GetPitch() const override;
  units::angle::radian_t GetRoll() const override;
  bool IsValidForLane(int64_t laneId) const override;
  RoadObjectType GetType() const override;
  units::length::meter_t GetLength() const override;
  units::length::meter_t GetWidth() const override;
  bool IsContinuous() const override;
  std::string GetName() const override;

private:
  RoadInterface* road;
  const RoadObjectSpecification object;

public:
  RoadObject(const RoadObject&) = delete;
  RoadObject(RoadObject&&) = delete;
  RoadObject& operator=(const RoadObject&) = delete;
  RoadObject& operator=(RoadObject&&) = delete;
  ~RoadObject() override = default;
};
