/********************************************************************************
 * Copyright (c) 2016-2018 ITK Engineering GmbH
 *               2017-2019 in-tech GmbH
 *               2023 Bayerische Motoren Werke Aktiengesellschaft (BMW AG)
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 ********************************************************************************/

//-----------------------------------------------------------------------------
/** @file  stochastics.cpp */
//-----------------------------------------------------------------------------

#include "stochastics.h"

#include <new>
#include <stdexcept>
#include <string>

#include "include/callbackInterface.h"
#include "include/stochasticsInterface.h"
#include "opSimulation/modules/Stochastics/stochastics_global.h"
#include "stochastics_implementation.h"

/// The version of the current module - has to be incremented manually
const std::string Version = "0.0.1";
static const CallbackInterface *Callbacks = nullptr;  // NOLINT[cppcoreguidelines-avoid-non-const-global-variables]

/**
 * @brief   dll-function to obtain the version of the current module
 *
 * @return  Version of the current module
 */
extern "C" STOCHASTICS_SHARED_EXPORT const std::string &OpenPASS_GetVersion()
{
  return Version;
}

/**
 * @brief   dll-function to create an instance of the module
 *
 * @param[in]   callbacks   Pointer to the callbacks
 * @return  A pointer to the created module instance
 */
extern "C" STOCHASTICS_SHARED_EXPORT StochasticsInterface *OpenPASS_CreateInstance(CallbackInterface *callbacks)
{
  Callbacks = callbacks;

  try
  {
    return (StochasticsInterface *)(new (std::nothrow) StochasticsImplementation(callbacks));
  }
  catch (const std::exception &ex)
  {
    if (Callbacks != nullptr)
    {
      Callbacks->Log(CbkLogLevel::Error, __FILE__, __LINE__, ex.what());
    }
    return nullptr;
  }
  catch (...)
  {
    if (Callbacks != nullptr)
    {
      Callbacks->Log(CbkLogLevel::Error, __FILE__, __LINE__, "unexpected exception");
    }

    return nullptr;
  }
}

/**
 * @brief   dll-function to destroy/delete an instance of the module
 *
 * @param[in]   implementation  The instance that should be freed
 */
extern "C" STOCHASTICS_SHARED_EXPORT void OpenPASS_DestroyInstance(StochasticsInterface *implementation)
{
  delete implementation;  // NOLINT(cppcoreguidelines-owning-memory)
}
