#!/bin/bash

################################################################################
# Copyright (c) 2021 in-tech GmbH
#               2023-2025 Bayerische Motoren Werke Aktiengesellschaft (BMW AG)
#
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# http://www.eclipse.org/legal/epl-2.0.
#
# SPDX-License-Identifier: EPL-2.0
################################################################################

################################################################################
# This script executes end-to-end tests
################################################################################

if [[ "$(uname)" != "Linux" ]]; then
  echo "Skipping E2E tests on non-Linux platform"
  exit 0
fi
SCRIPT_DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"
OP_REPO_DIR="${OP_REPO_DIR:=$SCRIPT_DIR/../../..}"

if [[ -z "${GECCO_DIR}" ]]; then
  # If not defined, set a default value
  GTGEN_DIR="$SCRIPT_DIR/../../../../gtgen_cli"
  GECCO_DIR="$SCRIPT_DIR/../../../../gecco"
  echo "WARN: GECCO_DIR variable is not set. Setting it to:
        GECCO_DIR is ${GECCO_DIR}"
fi

GTGEN_VERSION="add-new-commandline-parameters"
GTGEN_CLI_PATH="$GTGEN_DIR/gt-gen-simulator-${GTGEN_VERSION}/bazel-out/k8-opt/bin/Cli"
venv_python_executable="${OP_REPO_DIR}/.env/bin/python3"

commonFolder=$GECCO_DIR/gecco/examples/Common
commonFolder=$(cd $(dirname "$commonFolder") || exit; pwd)/$(basename "$commonFolder")

resourcePath=$GECCO_DIR/gecco/examples/Configurations
resourcePath=$(cd $(dirname "$resourcePath") || exit; pwd)/$(basename "$resourcePath")

allowedWarningsFiles=$OP_REPO_DIR/sim/tests/endToEndTests/allowed_end_to_end_warnings.txt

pluginsPath="$GECCO_DIR"/gecco/modules
echo "============================== Running test_gtgen_e2e =============================="

${venv_python_executable} -m pytest -vvv --simulation="${GTGEN_CLI_PATH}/gtgen_cli" --mutual="${commonFolder}" --resources="${resourcePath}" --plugins-path="${pluginsPath}" --allowed-warnings="${allowedWarningsFiles}" --report-path="${GTGEN_CLI_PATH}"/testreport "${OP_REPO_DIR}"/sim/tests/endToEndTests/test_end_to_end.json
exit_code=$?

mkdir -p "$SCRIPT_DIR/../../../../dist" || exit 1
mkdir -p "$SCRIPT_DIR"/../../../../dist/opSimulation || exit 1
cp -r "${GTGEN_CLI_PATH}"/artifacts "$SCRIPT_DIR"/../../../../dist/opSimulation || exit 1
cp -r "${GTGEN_CLI_PATH}"/testreport "$SCRIPT_DIR"/../../../../dist/opSimulation || exit 1

exit $exit_code