#!/usr/bin/env bash

################################################################################
# Copyright (c) 2022-2024 Bayerische Motoren Werke Aktiengesellschaft (BMW AG)
#
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# http://www.eclipse.org/legal/epl-2.0.
#
# SPDX-License-Identifier: EPL-2.0
################################################################################

################################################################################
# This script checks the clang format for the modified/added/copied files
################################################################################

MYDIR="$(dirname "$(readlink -f $0)")"
cd "$MYDIR/../../.." || exit 1
exit_code=0

# Skipping the check for windows
if [[ "${OSTYPE}" = "msys" ]]; then
  echo "Since the clangformat check is not enabled on Windows systems (for now), it is being skipped."
  exit 0
fi

clang-format-15 --version

for file in $(find . -type f \( -name "*.h" -o -name "*.cpp" \) ! -path "./sim/deps/*" ! -path "./build/*" ! -path "./gui/*" ! -path "./.git/*" ! -path "./.env/*"); do
    if $(clang-format-15 -output-replacements-xml $file | grep -c "<replacement " > /dev/null); then
        if [[ ! "$file" =~ version\.h$ && ! "$file" =~ version\.cpp$ ]]; then # filtering version.h as it is autogenerated
            echo "ERROR [clang-format]: $file is not formatted properly"
            clang-format-15 --style=file --dry-run --Werror "$file"
            exit_code=1
        fi
    fi
done

if [ $exit_code -eq 0 ]; then
    echo "SUCCESS [clang-format]: Files are correctly formatted"
fi

exit $exit_code
