#!/bin/bash

################################################################################
# Copyright (c) 2025 Bayerische Motoren Werke Aktiengesellschaft (BMW AG)
#
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# http://www.eclipse.org/legal/epl-2.0.
#
# SPDX-License-Identifier: EPL-2.0
################################################################################

################################################################################
# This Script pull the optestrunner binaries and installs them
################################################################################

SCRIPT_DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"
if [[ -z "${GTGEN_DIR}" ]]; then
  # If not defined, set a default value
  GTGEN_DIR="$SCRIPT_DIR/../../../../gtgen_cli"
  echo "WARN: GTGEN_DIR variable is not set. Setting it to:
        GTGEN_DIR is ${GTGEN_DIR}"
fi

mkdir -p "$GTGEN_DIR" && cd "$GTGEN_DIR" || exit 1
rm -rf -- * || { echo "Failed to clean $GTGEN_DIR directory"; exit 1; }
bazel --version || { echo "Unable to run bazel"; exit 1; }

# ToDo: Make it generic
GTGEN_VERSION="add-new-commandline-parameters"
GTGEN_URL="https://gitlab.eclipse.org/eclipse/openpass/gt-gen-simulator/-/archive/${GTGEN_VERSION}/gt-gen-simulator-${GTGEN_VERSION}.tar.gz"
GTGEN_CLI_PATH="$GTGEN_DIR/gt-gen-simulator-${GTGEN_VERSION}/bazel-out/k8-opt/bin/Cli"
CACHEDIR=$GTGEN_DIR

# This override the cache folder of bazel
export TEST_TMPDIR="${CACHEDIR}"
export BAZELISK_HOME="${CACHEDIR}"

#download gtgen
download_gtgen() {
  echo "============================== Downloading gt-gen-simulator =============================="
  wget $GTGEN_URL || { echo "Unable to download  gt-gen-simulator"; exit 1; }
  tar -xzvf gt-gen-simulator-${GTGEN_VERSION}.tar.gz || { echo "Unable to checkout $GTGEN_VERSION"; exit 1; }
  echo "============================== Cloning Successful =============================="
}

#build gtgen
build_gtgen() {
  echo "============================== Building gt-gen-simulator =============================="
  cd "$GTGEN_DIR"/gt-gen-simulator-${GTGEN_VERSION} || { echo "Unable to change directory into  gt-gen-simulator"; exit 1; }
  bazel build --config=gt_gen_release //Cli/... //Simulator/... || { echo "Unable to build  gt-gen-simulator"; exit 1; }
  echo "============================== Extraction Successful =============================="
}

show_version() {
  echo "============================== Version info of gt-gen-simulator =============================="
  cd "$GTGEN_CLI_PATH" || { echo "Unable to change directory into  gt-gen-simulator executable"; exit 1; }
  ./gtgen_cli --version || { echo "Unable to execute  gt-gen-simulator"; exit 1; }
}

download_and_build_gtgen() {
  download_gtgen
  build_gtgen
  show_version
}

download_and_build_gtgen