#!/bin/bash

################################################################################
# Copyright (c) 2025 Bayerische Motoren Werke Aktiengesellschaft (BMW AG)
#
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# http://www.eclipse.org/legal/epl-2.0.
#
# SPDX-License-Identifier: EPL-2.0
################################################################################

################################################################################
# This Script pull the gtgen binaries and installs them
################################################################################

SCRIPT_DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"
#ToDo: Set this variable at the top level
if [[ -z "${OP_GTGEN_DIR}" ]]; then
  # If not defined, set a default value
  OP_GTGEN_DIR="$SCRIPT_DIR/../../../../gtgen_cli"
  echo "WARN: OP_GTGEN_DIR variable is not set. Setting it to:
        OP_GTGEN_DIR is ${OP_GTGEN_DIR}"
fi

mkdir -p "$OP_GTGEN_DIR" && cd "$OP_GTGEN_DIR" || exit 1
rm -rf -- * || { echo "Failed to clean $OP_GTGEN_DIR directory"; exit 1; }

GTGEN_SIM_FILE=$(basename "$GTGEN_SIM_URL")
GTGEN_CLI_FILE=$(basename "$GTGEN_CLI_URL")

#download gtgen_core and gtgen_cli
download_deb() {
  echo "============================== Downloading $GTGEN_SIM_FILE =============================="
  wget --no-check-certificate "${GTGEN_SIM_URL}" || { echo "Unable to download gtgen core $GTGEN_SIM_FILE from ddad artifactory"; exit 1; }
  echo "============================== Download Successful =============================="
  echo "============================== Downloading $GTGEN_CLI_FILE =============================="
  wget --no-check-certificate "${GTGEN_CLI_URL}" || { echo "Unable to download gtgen cli $GTGEN_CLI_FILE from ddad artifactory"; exit 1; }
  echo "============================== Download Successful =============================="
}

#Installs gtgen_core and gtgen_cli
install_deb() {
  echo "============================== Installing $GTGEN_SIM_FILE =============================="
  dpkg-deb -X ./"$GTGEN_SIM_FILE" "$OP_GTGEN_DIR" || { echo "Unable to install gtgen simulator $GTGEN_SIM_FILE package"; exit 1; }
  echo "============================== Installation Successful =============================="
  echo "============================== Installing $GTGEN_SIM_FILE =============================="
  dpkg-deb -X ./"$GTGEN_CLI_FILE" "$OP_GTGEN_DIR" || { echo "Unable to install gtgen cli $GTGEN_CLI_FILE package"; exit 1; }
  echo "============================== Installation Successful =============================="
}

install_and_verify_deb() {
  download_deb
  install_deb
  mv "$OP_GTGEN_DIR"/opt/gtgen_cli/* . || { echo "Unable to move gtgen cli to $OP_GTGEN_DIR directory"; exit 1; }
  mv "$OP_GTGEN_DIR"/opt/gtgen_core/lib/* . || { echo "Unable to move gtgen core to $OP_GTGEN_DIR directory"; exit 1; }
  ./gtgen_cli -v || { echo "Unable to execute gtgen cli with '-v' option"; exit 1; }
}

install_and_verify_deb